/*===========================================================================*/
/* SEED reader     |               parse_type50              |  station header */
/*===========================================================================*/
/*
	Name:		parse_type50
	Purpose:	parse a station id (type 50) blockette into a 
				globally-available table
	Usage:		void parse_type50 ();
				char *blockette;
				parse_type50 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
				type50_head - 1st entry in table; defined in structures.h,
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_abbrevdic
	Calls to:	parse_int - get an integer from a blockette 
				parse_double - get a double from a blockette 
				parse_nchar - get a fixed-length string from a blockette 
				parse_varlstr - get a variable-length string from a blockette
	Algorithm:	parse the blockette
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/04/88  Dennis O'Neill  replaced "~" by STRINGTERM in
				                          parse_varlstr calls
				11/21/88  Dennis O'Neill  Production release 1.0
				08/19/89  Dennis O'Neill  streamlined linked list construction
				09/17/89  Dennis O'Neill  removed number_channels and 
				                          number_comments fields
				04/26/93  Allen Nance     Added network code for v2.3
*/

#include "rdseed.h"

struct type50 *parse_type50 (blockette)
	char *blockette;								/* ptr to start of blockette */
{
	struct type50 *type50; 							/* genl member of table */
	char *blockette_ptr;						/* ptr to inside blockette */

	if ((type50 = (struct type50 *) malloc(sizeof(struct type50))) != NULL)
		bzero (type50, sizeof(struct type50));
	else return (type50);
	    
	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;

/*                 +=======================================+                 */
/*=================|   recover information from the type50   |=================*/
/*                 +=======================================+                 */

	/* recover station symbol */
	type50->station = parse_nchar (&blockette_ptr, 5);

	/* recover latitude */
	type50->latitude = parse_double (&blockette_ptr, 10);

	/* recover longitude */
	type50->longitude = parse_double (&blockette_ptr, 11);

	/* recover elevation */
	type50->elevation = parse_double (&blockette_ptr, 7);

	/* recover number of channels - old spec, removed */
	/* type50->number_channels = parse_int (&blockette_ptr, 4); */
	type50->reserved1 = parse_int (&blockette_ptr, 4);

	/* recover number of station comment blockettes - old spec, removed */
	/* type50->number_comments = parse_int (&blockette_ptr, 3); */
	type50->reserved2 = parse_int (&blockette_ptr, 3);

	/* recover site name */
	type50->name = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover network/station owner name code (see generic abbrev dict) */
	type50->owner_code = parse_int (&blockette_ptr, 3);

	/* recover 32-bit byte significance order */
	type50->longword_order = parse_int (&blockette_ptr, 4);

	/* recover 16-bit byte significance order */
	type50->word_order = parse_int (&blockette_ptr, 2);

	/* recover starting effective date */
	type50->start = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover ending effective date, allowing for zero length */
	if (*blockette_ptr != '~')
		type50->end = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		blockette_ptr += 1;
		type50->end = NULL;
	}

	/* recover update flag */
	type50->update = parse_nchar (&blockette_ptr, 1);

	/* recover update flag */
	if (seed_version >= 2.3)
	{
		type50->network_code = parse_nchar (&blockette_ptr, 2);
	}
	else
	{
		type50->network_code = NULL;
	}

	return (type50);
}

void free_type50 (type50)
    struct type50 *type50;
{
    if (type50 == NULL) return;
    if (type50->station)	free(type50->station);
    if (type50->name)		free(type50->name);
    if (type50->start)		free(type50->start);
    if (type50->end)		free(type50->end);
    if (type50->update)		free(type50->update);
    if (type50->network_code)	free(type50->network_code);
    free (type50);
}
